#NOTICE: This software script is protected by the Copyright Act of 1976, 17 U.S.C. §§ 101-810, as amended.  Rights reserved.  Please contact The Brigham and Women’s Hospital, Inc. for further information.                                                          mIMT-visHTS 
#mIMNT.r Version: 1.0.0                                         
#Date:  2nd Mar. 2015                                  
##############################################################################
# Execution notes:                                          		         #
# 1/ place this script in the same folder of the input files;  		         #
# 2/ change the variables f_name1 /2 /3 with yours;		                     #
# 3/ For R-2.13 and above install mclust package via biocLite of Bioconductor#
# source("http://www.bioconductor.org/biocLite.R")                           #
# biocLite("mclust") 														 #
# This script is optimized for three input datasets with a maximum output of # 
# 20 clusters                                                      	         #
##############################################################################

options(warn=1)

library(graphics)
library(mclust)
library(ggplot2)
library(gridExtra)
library(stringr)

###################
#Setting up VALUES#
###################

nClusters <-15 #set max number of cluster based on BIC index
tot_col<-6 #Number of time points
nCluster_Range <- 2:nClusters
tmtcols <- 1:tot_col # TMT sixplex
xlabs=c("0","8","12","24","48","72")
f_name1<-c("HI_102112_M0_0-72hr_OFFGEL_Fr1-24-Sequest_EDITED.csv")
f_name2<-c("HI_102112_M1_200microgram_postOFFGEL_final_EDITED.csv")
f_name3<-c("HI_102112_M2_200microgram_postOFFGEL_final2_EDITED.csv")

###################
#Directory Outputs#
###################

wdir<-getwd()
res<-paste(wdir,"/mIMToutput",sep="")
dir.create(res, showWarnings = FALSE)

##################
#Directory Inputs#
##################

CSV1=read.csv(file=f_name1, header=T, sep=",")
CSV2=read.csv(file=f_name2, header=T, sep=",")
CSV3=read.csv(file=f_name3, header=T, sep=",")

clustering=1 # clustering=1 for clustering : clustering=0 for no clustering

######################################
#GENERAL SETTINGS -NOT TO BE MODIFIED#
######################################

#############################
#LOADING & DATA MANIPULATION#
#############################

Dataset1 <- subset(CSV1, X..Unique.Peptides >1, select = c(Channel_1, Channel_2, Channel_3, Channel_4, Channel_5, Channel_6, Accession, Description))
Dataset2 <- subset(CSV2, X..Unique.Peptides >1, select = c(Channel_1, Channel_2, Channel_3, Channel_4, Channel_5, Channel_6, Accession, Description))
Dataset3 <- subset(CSV3, X..Unique.Peptides >1, select = c(Channel_1, Channel_2, Channel_3, Channel_4, Channel_5, Channel_6, Accession, Description))

#Remove missing data
subsets_ratios1 <- na.omit(Dataset1)
subsets_ratios2 <- na.omit(Dataset2)
subsets_ratios3 <- na.omit(Dataset3)

#Sum normalization calculations
vect_sum1=as.matrix(rowSums(subsets_ratios1[1:tot_col]))
vect_sum2=as.matrix(rowSums(subsets_ratios2[1:tot_col]))
vect_sum3=as.matrix(rowSums(subsets_ratios3[1:tot_col]))
colnames(vect_sum1)<-c("rowsum")
colnames(vect_sum2)<-c("rowsum")
colnames(vect_sum3)<-c("rowsum")
s1=subsets_ratios1[1:6] /vect_sum1
s2=subsets_ratios2[1:6] /vect_sum2
s3=subsets_ratios3[1:6] /vect_sum3

#Sum normalization
DSSN1 <- cbind(s1, subsets_ratios1["Accession"],subsets_ratios1["Description"])
DSSN2 <- cbind(s2, subsets_ratios2["Accession"],subsets_ratios2["Description"])
DSSN3 <- cbind(s3, subsets_ratios3["Accession"],subsets_ratios3["Description"])

#Gene ID extraction and dataset tagging
pattern<-"GN=(\\S+)"
g_fromD1<-str_extract(DSSN1$Description,perl(pattern))
g_fromD2<-str_extract(DSSN2$Description,perl(pattern))
g_fromD3<-str_extract(DSSN3$Description,perl(pattern))

gID_D1a<-str_split_fixed(g_fromD1,"GN=",n=2)
gID_D1a<-as.vector(gID_D1a[,2])

gID_D2a<-str_split_fixed(g_fromD2,"GN=",n=2)
gID_D2a<-as.vector(gID_D2a[,2])

gID_D3a<-str_split_fixed(g_fromD3,"GN=",n=2)
gID_D3a<-as.vector(gID_D3a[,2])

gID_D1<-as.matrix(replace(gID_D1a,gID_D1a=="","GENEID_NOT_FOUND"))
gID_D2<-as.matrix(replace(gID_D2a,gID_D2a=="","GENEID_NOT_FOUND"))
gID_D3<-as.matrix(replace(gID_D3a,gID_D3a=="","GENEID_NOT_FOUND"))

#Addition of the gene ID column
DSSN1["geneID"]<-gID_D1
DSSN2["geneID"]<-gID_D2
DSSN3["geneID"]<-gID_D3

#Removal of entries without a gene ID
new1<-DSSN1[!(DSSN1$geneID %in% "GENEID_NOT_FOUND"),]
new2<-DSSN2[!(DSSN2$geneID %in% "GENEID_NOT_FOUND"),]
new3<-DSSN3[!(DSSN3$geneID %in% "GENEID_NOT_FOUND"),]

DSSN1<-new1
DSSN2<-new2
DSSN3<-new3

#Text tag IDs
id_DS1<-c("_id_DS1")
id_DS2<-c("_id_DS2")
id_DS3<-c("_id_DS3")
gID_D1_SuperDS=paste(DSSN1$geneID,id_DS1,sep="")
gID_D2_SuperDS=paste(DSSN2$geneID,id_DS2,sep="")
gID_D3_SuperDS=paste(DSSN3$geneID,id_DS3,sep="")
DS1_SC=DSSN1
DS2_SC=DSSN2
DS3_SC=DSSN3

DS1_SC["geneID"]<-gID_D1_SuperDS
DS2_SC["geneID"]<-gID_D2_SuperDS
DS3_SC["geneID"]<-gID_D3_SuperDS

#Remove any potential duplicated genes 
DSSN1=DSSN1[!duplicated(DSSN1$geneID),]
DSSN2=DSSN2[!duplicated(DSSN2$geneID),]
DSSN3=DSSN3[!duplicated(DSSN3$geneID),]
DS1_SC=DS1_SC[!duplicated(DS1_SC$geneID),]
DS2_SC=DS2_SC[!duplicated(DS2_SC$geneID),]
DS3_SC=DS3_SC[!duplicated(DS3_SC$geneID),]

#Merge into super dataset
SuperDS <-rbind(DS1_SC,DS2_SC,DS3_SC)

#################
#END LODING DATA#      
#################

if(clustering==1){
 
############
#Clustering#
############

out<-paste(res,"/", "BIC_index.pdf", sep="")
pdf(file = out)

print('Clustering Super Dataset!')       

#Super Cluster
Sds_comp<-SuperDS
Sds_cl <- Sds_comp[,1:tot_col]
f <- Sds_comp$geneID
rownames(Sds_cl)<-f
mc_Sds <- Mclust(Sds_cl[,tmtcols][,],G=nCluster_Range)
summSds<-summary(mc_Sds,parameters=T)
print(summSds)
plot.Mclust(mc_Sds, what="BIC")
dev.off()

#Excel output#

#Super DataSet
desc <- as.character(Sds_comp[match(rownames(Sds_cl), Sds_comp$geneID),"Description"])
xls_Sds <- data.frame( as.character(desc), mc_Sds$class, as.matrix(Sds_cl[,tmtcols]) )
colnames(xls_Sds) <- c( "Description", "Cluster", paste("TMT", tmtcols,sep=""))
write.csv(xls_Sds, file = paste(res,"/", "SuperDS_Cluster.csv", sep=""))
              
tmpfile=read.csv(file=paste(res,"/","SuperDS_Cluster.csv", sep=""))
unlink(paste(res,"/", "SuperDS_Cluster.csv", sep=""))
colnames(tmpfile)<-c("geneID", "Description", "Cluster", paste("TMT", tmtcols,sep=""))
write.csv(tmpfile, file = paste(res,"/", "SuperDS_Cluster.csv", sep=""))

##End clustering###

##Plot the clusters
input_clust_out = paste(res, "/","SuperDS_Cluster.csv", sep="")
ico = read.csv(file(input_clust_out), header=T, sep=",")
       
#Plot parameters
Xaxis_lab=0.8
Yaxis_lab=0.8
axis_labels=1
title_size=1
base_size <- 10

ticks <- data.frame (t = c(1,2,3,4,5,6))

out<-paste(res,"/","SuperCluster",nClusters,".pdf",sep="")
pdf(file = out)

SuperDS_List <- list()
for (i in 1:mc_Sds$G){
counter<-i
cl_Sds<-as.matrix(subset(ico, Cluster %in% counter, select = c(TMT1,TMT2,TMT3,TMT4,TMT5,TMT6)))
nlines<-nrow(cl_Sds)
df_lines_Sds <- data.frame(expand.grid(x = tmtcols, y = 1:nlines))
# create a vector with the TMT values in a column
z_levels_Sds=as.numeric(t(cl_Sds[1:nlines, tmtcols]))
data_Sds<-cbind(df_lines_Sds,z_levels_Sds)


#Cluster color assignments
    if(i==0){
        set_col<-c("#000000")}
    if(i==1){
        set_col<-c("#BEBEBE")}
    if(i==2){
        set_col<-c("#00008B")}
    if(i==3){
        set_col<-c("#1E90FF")}
    if(i==4){
        set_col<-c("#104E8B")}
    if(i==5){
        set_col<-c("#FF0000")}
    if(i==6){
        set_col<-c("#8B0000")}
    if(i==7){
        set_col<-c("#9ACD32")}
    if(i==8){
        set_col<-c("#008B00")}
    if(i==9){
        set_col<-c("#00FFFF")}
    if(i==10){
        set_col<-c("#008B8B")}
    if(i==11){
        set_col<-c("#0000FF")}
    if(i==12){
        set_col<-c("#8B0A50")}
    if(i==13){
        set_col<-c("#FFFF00")}
    if(i==14){
        set_col<-c("#8B7500")}
    if(i==15){
        set_col<-c("#00FF00")}
    if(i==16){
        set_col<-c("#FF69B4")}
    if(i==17){
        set_col<-c("#A020F0")}
    if(i==18){
        set_col<-c("#FFA500")}
    if(i==19){
        set_col<-c("#FF00FF")}
    if(i==20){
        set_col<-c("#8B008B")}

 
    SuperDS_List[[i]]<-ggplot(data_Sds, aes(x=x, y=z_levels_Sds, group=y)) + geom_line(colour = set_col) +
    geom_hline(yintercept=1/tot_col, color="red", linetype="dashed")+
    labs(x = "", y = "") +
    labs(title =  counter) +
    scale_x_continuous(breaks=c(ticks$t), labels=xlabs) +
    theme(axis.ticks = element_blank(),
    axis.text.x = element_text(size = base_size *  0.8, angle = 0, hjust = 0),
    axis.text.y = element_text(size = base_size *  0.8, angle = 0, hjust = 0)) +
    theme(plot.title = element_text(size = base_size *0.8, colour = "black"))
   
    
}
do.call(grid.arrange,SuperDS_List)
dev.off()

}

